@@ -43,7 +43,6 @@ use crate::Connection;
 use crate::ConnectionError;
 
 use crate::crypto;
-use crate::octets;
 use crate::packet;
 
 const TLS1_3_VERSION: u16 = 0x0304;
@@ -124,7 +123,8 @@ struct SSL_QUIC_METHOD {
 }
 
 lazy_static::lazy_static! {
-    static ref QUICHE_EX_DATA_INDEX: c_int = unsafe {
+    /// BoringSSL Extra Data Index for Quiche Connections
+    pub static ref QUICHE_EX_DATA_INDEX: c_int = unsafe {
         SSL_get_ex_new_index(0, ptr::null(), ptr::null(), ptr::null(), ptr::null())
     };
 }
@@ -154,6 +154,16 @@ impl Context {
         }
     }
 
+    #[cfg(feature = "boringssl-boring-crate")]
+    pub fn from_boring(ssl_ctx: boring::ssl::SslContext) -> Context {
+        use foreign_types_shared::ForeignType;
+
+        let mut ctx = Context(ssl_ctx.into_ptr() as _);
+        ctx.set_session_callback();
+
+        ctx
+    }
+
     pub fn new_handshake(&mut self) -> Result<Handshake> {
         unsafe {
             let ssl = SSL_new(self.as_mut_ptr());
@@ -923,7 +933,6 @@ extern fn select_alpn(
     3 // SSL_TLSEXT_ERR_NOACK
 }
 
-#[no_mangle]
 extern fn new_session(ssl: *mut SSL, session: *mut SSL_SESSION) -> c_int {
     let conn =
         match get_ex_data_from_ptr::<Connection>(ssl, *QUICHE_EX_DATA_INDEX) {
